require 'model'
require 'util'

class App
  
  attr_accessor :user, :records
  
  def initialize
    @case_records = []
  end

  def current_record
    @case_records.last
  end

  def current_visit
    current_record.visits.last
  end

  def current_audit
    current_record.audits.last
  end


  ### Actions taken on behalf of a user. The app's primary interface.

  def login(keys)
    @user = keys['login']
    :main_page
  end

  def want_new_case_form(keys)
    :gather_case_input_page
  end
  
  def record_case(keys)
    @case_records << CaseRecord.new
    current_record.client = keys['client']
    current_record.clinic_id = keys['clinic_id']
    :case_display_page
  end

  def want_add_visit_form(keys)
    :gather_visit_input_page
  end

  def record_visit(keys)
    current_record.visits << Visit.new
    current_visit.diagnosis = keys['diagnosis']
    current_visit.charges = keys['charges']
    current_visit.comments = keys['comments']
    :case_display_page
  end

  def want_add_audit_form(keys)
      :gather_audit_input_page
  end

  def record_audit(keys)
    current_record.audits << Audit.new
    current_audit.auditor = keys['auditor']
    current_audit.variance = keys['variance']
    :case_display_page
  end


  ### Filling out Renderer templates

  def fill(template_name, *args, &block)
    self.send("fill_#{template_name}", *args, &block)
  end

  def checked(action_name)
    assert(respond_to?(action_name),
           "#{action_name} is not a defined action.")
    action_name
  end


  def fill_template_for_want_add_audit_form(&block)
    return unless current_record.accepts_more_audits?

    block.call(checked(:want_add_audit_form))
  end

  def fill_template_for_help_link(topic, &block)
    assert(has_help_for?(topic),
           "Creating link to nonexistent help topic '#{topic}'.")
    block.call(checked(:help))
  end



  ### Delivering help pages

  def help(topic)
    assert(has_help_for?(topic), "There is no help for #{topic}.")
    IO.readlines(path_to(topic)).join($/)
  end

  def has_help_for?(topic)
    File.exist?(path_to(topic))
  end

  def path_to(topic)
    "help/#{topic}"
  end

end
