require 'webrick' 
require 'xmlrpc/server'
require 'paths'
require 'pp'
require 'app'
require 'renderer'

include WEBrick

class AppWebInterface

  def initialize(app)
    @server = HTTPServer.new(:Port => 8080)
    @app = app
    @renderer = Renderer.new(@app)
    @dispatched = []
    @current_page_name = :no_page_yet
    install_UI_servlets
    install_testing_servlets
    install_utility_servlets
    trap("INT"){ @server.shutdown } 
    @server.start
  end

  def install_generic_proc(mount_point, &block)
    @server.mount_proc(mount_point,
                       proc { | request, response |
                         response.body = block.call(request, @app)
                         raise HTTPStatus::OK
                       })
  end

  def install_command(command)
    install_generic_proc("/#{command}") { | request, app |
      @current_page_name = dispatch(command, request.query)
      render(@current_page_name)
    }
  end

  def dispatch(command, args)
    @dispatched << [command, args]
    @app.send(command, args)
  end

  def render(page_name)
    @current_xhtml = @renderer.send(page_name)
  end



  def install_UI_servlets
    install_generic_proc('/') { | request, app |
      assert(request.path=='/', "Unknown action #{request.path}.")
      render(:beginning_page)
    }
    install_command(:login)
    install_command(:want_new_case_form)
    install_command(:record_case)
    install_command(:want_add_visit_form)
    install_command(:want_add_audit_form)
    install_command(:record_visit)
    install_command(:record_audit)
  end

  def install_utility_servlets
    install_generic_proc('/ping') { | request, app |
      render(:ping_page)
    }

    install_generic_proc('/refresh') { | request, app |
      render(@current_page_name)
    }

    # Because help pages are state-preserving popups, they don't go
    # through the same channel as the normal app. The HTML is not
    # preserved for refreshing, etc.
    install_generic_proc('/help') { | request, app |
      @app.help(request.query['topic'])
    }
  end


  def install_testing_servlets
    dumper_servlet = XMLRPC::WEBrickServlet.new
    @server.mount("/side-channel", dumper_servlet)

    dumper_servlet.add_handler("snapshot") {
      retval = '' 
      PP.pp(@dispatched, retval)
      retval
    }

    dumper_servlet.add_handler("replay") { | command_string |
      command_descriptions = eval(command_string)
      command_descriptions.each { | one |
        @current_page_name = dispatch(*one)
      }
      'return value irrelevant but it cannot be nil'
    }

  end


end

if __FILE__ == $0
  AppWebInterface.new(App.new)
end
